@@ -1,34 +1,35 @@
 // Copyright 2019-2022 ChainSafe Systems
 // SPDX-License-Identifier: Apache-2.0, MIT
 
+use core::fmt;
 use std::cell::RefCell;
 use std::collections::{BTreeMap, HashMap, VecDeque};
-use std::convert::TryInto;
 
 use anyhow::anyhow;
-use cid::multihash::{Code, Multihash};
+use cid::multihash::{Code, MultihashDigest};
 use cid::Cid;
-use fvm_ipld_blockstore::MemoryBlockstore;
-use fvm_ipld_encoding::de::DeserializeOwned;
-use fvm_ipld_encoding::{Cbor, CborStore, RawBytes};
 use fvm_shared::actor::builtin::Type;
 use fvm_shared::address::{Address, Protocol};
+use fvm_shared::blockstore::{CborStore, MemoryBlockstore};
 use fvm_shared::clock::ChainEpoch;
 use fvm_shared::consensus::ConsensusFault;
 use fvm_shared::crypto::randomness::DomainSeparationTag;
 use fvm_shared::crypto::signature::Signature;
 use fvm_shared::econ::TokenAmount;
+use fvm_shared::encoding::de::DeserializeOwned;
+use fvm_shared::encoding::{blake2b_256, Cbor, RawBytes};
 use fvm_shared::error::ExitCode;
 use fvm_shared::piece::PieceInfo;
 use fvm_shared::randomness::Randomness;
 use fvm_shared::sector::{
-    AggregateSealVerifyProofAndInfos, RegisteredSealProof, SealVerifyInfo, WindowPoStVerifyInfo,
+    AggregateSealVerifyProofAndInfos, RegisteredSealProof, ReplicaUpdateInfo, SealVerifyInfo,
+    WindowPoStVerifyInfo,
 };
 use fvm_shared::version::NetworkVersion;
 use fvm_shared::{ActorID, MethodNum};
 
 use crate::runtime::{ActorCode, MessageInfo, Policy, Runtime, RuntimePolicy, Syscalls};
-use crate::{actor_error, ActorError, SYS_FORBIDDEN, SYS_ILLEGAL_ACTOR};
+use crate::{actor_error, ActorError};
 
 lazy_static! {
     pub static ref SYSTEM_ACTOR_CODE_ID: Cid = make_builtin(b"fil/test/system");
@@ -63,8 +64,9 @@ lazy_static! {
 
 const IPLD_RAW: u64 = 0x55;
 
+/// Returns an identify CID for bz.
 fn make_builtin(bz: &[u8]) -> Cid {
-    Cid::new_v1(IPLD_RAW, Multihash::wrap(0, bz).expect("name too long"))
+    Cid::new_v1(IPLD_RAW, Code::Identity.digest(bz))
 }
 
 pub struct MockRuntime {
@@ -127,10 +129,7 @@ impl Expectations {
         self.expect_verify_consensus_fault = None;
     }
     fn verify(&mut self) {
-        assert!(
-            !self.expect_validate_caller_any,
-            "expected ValidateCallerAny, not received"
-        );
+        assert!(!self.expect_validate_caller_any, "expected ValidateCallerAny, not received");
         assert!(
             self.expect_validate_caller_addr.is_none(),
             "expected ValidateCallerAddr {:?}, not received",
@@ -250,10 +249,10 @@ pub struct ExpectComputeUnsealedSectorCid {
     exit_code: ExitCode,
 }
 
-pub fn expect_abort_contains_message(
+pub fn expect_abort_contains_message<T: fmt::Debug>(
     expect_exit_code: ExitCode,
     expect_msg: &str,
-    res: Result<RawBytes, ActorError>,
+    res: Result<T, ActorError>,
 ) {
     let err = res.expect_err(&format!(
         "expected abort with exit code {}, but call succeeded",
@@ -275,16 +274,13 @@ pub fn expect_abort_contains_message(
     );
 }
 
-pub fn expect_abort(exit_code: ExitCode, res: Result<RawBytes, ActorError>) {
+pub fn expect_abort<T: fmt::Debug>(exit_code: ExitCode, res: Result<T, ActorError>) {
     expect_abort_contains_message(exit_code, "", res);
 }
 
 impl MockRuntime {
     fn require_in_call(&self) {
-        assert!(
-            self.in_call,
-            "invalid runtime invocation outside of method call",
-        )
+        assert!(self.in_call, "invalid runtime invocation outside of method call",)
     }
     fn put<C: Cbor>(&self, o: &C) -> Result<Cid, ActorError> {
         Ok(self.store.put_cbor(&o, Code::Blake2b256).unwrap())
@@ -299,6 +295,14 @@ impl MockRuntime {
         self.state()
     }
 
+    #[allow(dead_code)]
+    pub fn get_id_address(&self, address: &Address) -> Option<Address> {
+        if address.protocol() == Protocol::ID {
+            return Some(*address);
+        }
+        self.id_addresses.get(address).cloned()
+    }
+
     #[allow(dead_code)]
     pub fn expect_validate_caller_addr(&mut self, addr: Vec<Address>) {
         assert!(!addr.is_empty(), "addrs must be non-empty");
@@ -307,10 +311,7 @@ impl MockRuntime {
 
     #[allow(dead_code)]
     pub fn expect_verify_signature(&self, exp: ExpectedVerifySig) {
-        self.expectations
-            .borrow_mut()
-            .expect_verify_sigs
-            .push_back(exp);
+        self.expectations.borrow_mut().expect_verify_sigs.push_back(exp);
     }
 
     #[allow(dead_code)]
@@ -345,9 +346,7 @@ impl MockRuntime {
 
     #[allow(dead_code)]
     pub fn expect_compute_unsealed_sector_cid(&self, exp: ExpectComputeUnsealedSectorCid) {
-        self.expectations
-            .borrow_mut()
-            .expect_compute_unsealed_sector_cid = Some(exp);
+        self.expectations.borrow_mut().expect_compute_unsealed_sector_cid = Some(exp);
     }
 
     #[allow(dead_code)]
@@ -399,17 +398,14 @@ impl MockRuntime {
         send_return: RawBytes,
         exit_code: ExitCode,
     ) {
-        self.expectations
-            .borrow_mut()
-            .expect_sends
-            .push_back(ExpectedMessage {
-                to,
-                method,
-                params,
-                value,
-                send_return,
-                exit_code,
-            })
+        self.expectations.borrow_mut().expect_sends.push_back(ExpectedMessage {
+            to,
+            method,
+            params,
+            value,
+            send_return,
+            exit_code,
+        })
     }
 
     #[allow(dead_code)]
@@ -512,14 +508,10 @@ impl Runtime<MemoryBlockstore> for MockRuntime {
                 return Ok(());
             }
         }
-        self.expectations.borrow_mut().expect_validate_caller_addr = None;
-        return Err(ActorError::new(
-            SYS_FORBIDDEN,
-            format!(
+        expectations.expect_validate_caller_addr = None;
+        return Err(actor_error!(ErrForbidden;
                 "caller address {:?} forbidden, allowed: {:?}",
-                self.message().caller(),
-                &addrs
-            ),
+                self.message().caller(), &addrs
         ));
     }
     fn validate_immediate_caller_type<'a, I>(&mut self, types: I) -> Result<(), ActorError>
@@ -528,10 +520,7 @@ impl Runtime<MemoryBlockstore> for MockRuntime {
     {
         self.require_in_call();
         assert!(
-            self.expectations
-                .borrow_mut()
-                .expect_validate_caller_type
-                .is_some(),
+            self.expectations.borrow_mut().expect_validate_caller_type.is_some(),
             "unexpected validate caller code"
         );
 
@@ -545,11 +534,7 @@ impl Runtime<MemoryBlockstore> for MockRuntime {
         let types: Vec<Cid> = types.into_iter().map(find_by_type).collect();
         assert_eq!(
             &types,
-            self.expectations
-                .borrow_mut()
-                .expect_validate_caller_type
-                .as_ref()
-                .unwrap(),
+            self.expectations.borrow_mut().expect_validate_caller_type.as_ref().unwrap(),
             "unexpected validate caller code {:?}, expected {:?}",
             types,
             self.expectations.borrow_mut().expect_validate_caller_type
@@ -564,13 +549,8 @@ impl Runtime<MemoryBlockstore> for MockRuntime {
 
         self.expectations.borrow_mut().expect_validate_caller_type = None;
 
-        Err(ActorError::new(
-            SYS_FORBIDDEN,
-            format!(
-                "caller type {:?} forbidden, allowed: {:?}",
-                self.caller_type, types
-            ),
-        ))
+        Err(actor_error!(ErrForbidden; "caller type {:?} forbidden, allowed: {:?}",
+                self.caller_type, types))
     }
 
     fn current_balance(&self) -> TokenAmount {
@@ -613,18 +593,14 @@ impl Runtime<MemoryBlockstore> for MockRuntime {
 
     fn create<C: Cbor>(&mut self, obj: &C) -> Result<(), ActorError> {
         if self.state.is_some() {
-            panic!("state shouldn't have been set")
+            return Err(actor_error!(SysErrIllegalActor; "state already constructed"));
         }
         self.state = Some(self.store.put_cbor(obj, Code::Blake2b256).unwrap());
         Ok(())
     }
 
     fn state<C: Cbor>(&self) -> Result<C, ActorError> {
-        Ok(self
-            .store
-            .get_cbor(self.state.as_ref().unwrap())
-            .unwrap()
-            .unwrap())
+        Ok(self.store.get_cbor(self.state.as_ref().unwrap()).unwrap().unwrap())
     }
 
     fn transaction<C, RT, F>(&mut self, f: F) -> Result<RT, ActorError>
@@ -633,7 +609,7 @@ impl Runtime<MemoryBlockstore> for MockRuntime {
         F: FnOnce(&mut C, &mut Self) -> Result<RT, ActorError>,
     {
         if self.in_transaction {
-            panic!("recursive transaction")
+            return Err(actor_error!(SysErrIllegalActor; "nested transaction"));
         }
         let mut read_only = self.state()?;
         self.in_transaction = true;
@@ -656,10 +632,7 @@ impl Runtime<MemoryBlockstore> for MockRuntime {
     ) -> Result<RawBytes, ActorError> {
         self.require_in_call();
         if self.in_transaction {
-            return Err(ActorError::new(
-                SYS_ILLEGAL_ACTOR,
-                "side-effect within transaction".into(),
-            ));
+            return Err(actor_error!(SysErrIllegalActor; "side-effect within transaction"));
         }
 
         assert!(
@@ -671,12 +644,7 @@ impl Runtime<MemoryBlockstore> for MockRuntime {
             params
         );
 
-        let expected_msg = self
-            .expectations
-            .borrow_mut()
-            .expect_sends
-            .pop_front()
-            .unwrap();
+        let expected_msg = self.expectations.borrow_mut().expect_sends.pop_front().unwrap();
 
         assert!(expected_msg.to == to && expected_msg.method == method && expected_msg.params == params && expected_msg.value == value,
                 "expectedMessage being sent does not match expectation.\nMessage -\t to: {:?} method: {:?} value: {:?} params: {:?}\nExpected -\t {:?}",
@@ -685,7 +653,7 @@ impl Runtime<MemoryBlockstore> for MockRuntime {
         {
             let mut balance = self.balance.borrow_mut();
             if value > *balance {
-                return Err(actor_error!(SYS_SENDER_STATE_INVALID;
+                return Err(actor_error!(SysErrSenderStateInvalid;
                         "cannot send value: {:?} exceeds balance: {:?}",
                         value, *balance
                 ));
@@ -694,17 +662,14 @@ impl Runtime<MemoryBlockstore> for MockRuntime {
         }
 
         match expected_msg.exit_code {
-            ExitCode::OK => Ok(expected_msg.send_return),
+            ExitCode::Ok => Ok(expected_msg.send_return),
             x => Err(ActorError::new(x, "Expected message Fail".to_string())),
         }
     }
 
     fn new_actor_address(&mut self) -> Result<Address, ActorError> {
         self.require_in_call();
-        let ret = *self
-            .new_actor_addr
-            .as_ref()
-            .expect("unexpected call to new actor address");
+        let ret = *self.new_actor_addr.as_ref().expect("unexpected call to new actor address");
         self.new_actor_addr = None;
         Ok(ret)
     }
@@ -712,10 +677,7 @@ impl Runtime<MemoryBlockstore> for MockRuntime {
     fn create_actor(&mut self, code_id: Cid, actor_id: ActorID) -> Result<(), ActorError> {
         self.require_in_call();
         if self.in_transaction {
-            return Err(ActorError::new(
-                SYS_ILLEGAL_ACTOR,
-                "side-effect within transaction".into(),
-            ));
+            return Err(actor_error!(SysErrIllegalActor; "side-effect within transaction"));
         }
         let expect_create_actor = self
             .expectations
@@ -731,21 +693,14 @@ impl Runtime<MemoryBlockstore> for MockRuntime {
     fn delete_actor(&mut self, addr: &Address) -> Result<(), ActorError> {
         self.require_in_call();
         if self.in_transaction {
-            return Err(ActorError::new(
-                SYS_ILLEGAL_ACTOR,
-                "side-effect within transaction".into(),
-            ));
+            return Err(actor_error!(SysErrIllegalActor; "side-effect within transaction"));
         }
         let exp_act = self.expectations.borrow_mut().expect_delete_actor.take();
         if exp_act.is_none() {
             panic!("unexpected call to delete actor: {}", addr);
         }
         if exp_act.as_ref().unwrap() != addr {
-            panic!(
-                "attempt to delete wrong actor. Expected: {}, got: {}",
-                exp_act.unwrap(),
-                addr
-            );
+            panic!("attempt to delete wrong actor. Expected: {}, got: {}", exp_act.unwrap(), addr);
         }
         Ok(())
     }
@@ -792,11 +747,7 @@ impl Syscalls for MockRuntime {
                 hex::encode(plaintext)
             );
         }
-        let exp = self
-            .expectations
-            .borrow_mut()
-            .expect_verify_sigs
-            .pop_front();
+        let exp = self.expectations.borrow_mut().expect_verify_sigs.pop_front();
         if let Some(exp) = exp {
             if exp.sig != *signature || exp.signer != *signer || &exp.plaintext[..] != plaintext {
                 panic!(
@@ -824,88 +775,60 @@ impl Syscalls for MockRuntime {
     }
 
     fn hash_blake2b(&self, data: &[u8]) -> [u8; 32] {
-        blake2b_simd::Params::new()
-            .hash_length(32)
-            .to_state()
-            .update(data)
-            .finalize()
-            .as_bytes()
-            .try_into()
-            .expect("fixed array size")
+        blake2b_256(data)
     }
     fn compute_unsealed_sector_cid(
         &self,
         reg: RegisteredSealProof,
         pieces: &[PieceInfo],
     ) -> anyhow::Result<Cid> {
-        let exp = self
-            .expectations
-            .borrow_mut()
-            .expect_compute_unsealed_sector_cid
-            .take()
-            .ok_or_else(
-                || actor_error!(USR_ILLEGAL_STATE; "Unexpected syscall to ComputeUnsealedSectorCID"),
+        let exp =
+            self.expectations.borrow_mut().expect_compute_unsealed_sector_cid.take().ok_or_else(
+                || actor_error!(ErrIllegalState; "Unexpected syscall to ComputeUnsealedSectorCID"),
             )?;
 
         if exp.reg != reg {
-            return Err(anyhow!(actor_error!(USR_ILLEGAL_STATE;
+            return Err(anyhow!(actor_error!(ErrIllegalState;
                 "Unexpected compute_unsealed_sector_cid : reg mismatch"
             )));
         }
 
         if exp.pieces[..].eq(pieces) {
-            return Err(anyhow!(actor_error!(USR_ILLEGAL_STATE;
+            return Err(anyhow!(actor_error!(ErrIllegalState;
                 "Unexpected compute_unsealed_sector_cid : pieces mismatch"
             )));
         }
 
-        if exp.exit_code != ExitCode::OK {
-            return Err(anyhow!(ActorError::new(
-                exp.exit_code,
-                "Expected Failure".to_string(),
-            )));
+        if exp.exit_code != ExitCode::Ok {
+            return Err(anyhow!(ActorError::new(exp.exit_code, "Expected Failure".to_string(),)));
         }
         Ok(exp.cid)
     }
     fn verify_seal(&self, seal: &SealVerifyInfo) -> anyhow::Result<()> {
-        let exp = self
-            .expectations
-            .borrow_mut()
-            .expect_verify_seal
-            .take()
-            .ok_or_else(|| actor_error!(USR_ILLEGAL_STATE; "Unexpected syscall to verify seal"))?;
+        let exp =
+            self.expectations.borrow_mut().expect_verify_seal.take().ok_or_else(
+                || actor_error!(ErrIllegalState; "Unexpected syscall to verify seal"),
+            )?;
 
         if exp.seal != *seal {
-            return Err(anyhow!(
-                actor_error!(USR_ILLEGAL_STATE; "Unexpected seal verification"),
-            ));
+            return Err(anyhow!(actor_error!(ErrIllegalState; "Unexpected seal verification"),));
         }
-        if exp.exit_code != ExitCode::OK {
-            return Err(anyhow!(ActorError::new(
-                exp.exit_code,
-                "Expected Failure".to_string(),
-            )));
+        if exp.exit_code != ExitCode::Ok {
+            return Err(anyhow!(ActorError::new(exp.exit_code, "Expected Failure".to_string(),)));
         }
         Ok(())
     }
     fn verify_post(&self, post: &WindowPoStVerifyInfo) -> anyhow::Result<()> {
-        let exp = self
-            .expectations
-            .borrow_mut()
-            .expect_verify_post
-            .take()
-            .ok_or_else(|| actor_error!(USR_ILLEGAL_STATE; "Unexpected syscall to verify PoSt"))?;
+        let exp =
+            self.expectations.borrow_mut().expect_verify_post.take().ok_or_else(
+                || actor_error!(ErrIllegalState; "Unexpected syscall to verify PoSt"),
+            )?;
 
         if exp.post != *post {
-            return Err(anyhow!(
-                actor_error!(USR_ILLEGAL_STATE; "Unexpected PoSt verification"),
-            ));
+            return Err(anyhow!(actor_error!(ErrIllegalState; "Unexpected PoSt verification"),));
         }
-        if exp.exit_code != ExitCode::OK {
-            return Err(anyhow!(ActorError::new(
-                exp.exit_code,
-                "Expected Failure".to_string(),
-            )));
+        if exp.exit_code != ExitCode::Ok {
+            return Err(anyhow!(ActorError::new(exp.exit_code, "Expected Failure".to_string(),)));
         }
         Ok(())
     }
@@ -915,36 +838,22 @@ impl Syscalls for MockRuntime {
         h2: &[u8],
         extra: &[u8],
     ) -> anyhow::Result<Option<ConsensusFault>> {
-        let exp = self
-            .expectations
-            .borrow_mut()
-            .expect_verify_consensus_fault
-            .take()
-            .ok_or_else(
-                || actor_error!(USR_ILLEGAL_STATE; "Unexpected syscall to verify_consensus_fault"),
-            )?;
+        let exp = self.expectations.borrow_mut().expect_verify_consensus_fault.take().ok_or_else(
+            || actor_error!(ErrIllegalState; "Unexpected syscall to verify_consensus_fault"),
+        )?;
         if exp.require_correct_input {
             if exp.block_header_1 != h1 {
-                return Err(anyhow!(
-                    actor_error!(USR_ILLEGAL_STATE; "Header 1 mismatch")
-                ));
+                return Err(anyhow!(actor_error!(ErrIllegalState; "Header 1 mismatch")));
             }
             if exp.block_header_2 != h2 {
-                return Err(anyhow!(
-                    actor_error!(USR_ILLEGAL_STATE; "Header 2 mismatch")
-                ));
+                return Err(anyhow!(actor_error!(ErrIllegalState; "Header 2 mismatch")));
             }
             if exp.block_header_extra != extra {
-                return Err(anyhow!(
-                    actor_error!(USR_ILLEGAL_STATE; "Header extra mismatch"),
-                ));
+                return Err(anyhow!(actor_error!(ErrIllegalState; "Header extra mismatch"),));
             }
         }
-        if exp.exit_code != ExitCode::OK {
-            return Err(anyhow!(ActorError::new(
-                exp.exit_code,
-                "Expected Failure".to_string(),
-            )));
+        if exp.exit_code != ExitCode::Ok {
+            return Err(anyhow!(ActorError::new(exp.exit_code, "Expected Failure".to_string(),)));
         }
         Ok(exp.fault)
     }
@@ -955,6 +864,10 @@ impl Syscalls for MockRuntime {
         // TODO: Implement this if we need it. Currently don't have a need.
         todo!()
     }
+    fn verify_replica_update(&self, _replica: &ReplicaUpdateInfo) -> Result<(), anyhow::Error> {
+        // TODO: Implement this if we need it. Currently don't have a need.
+        todo!()
+    }
 }
 
 impl RuntimePolicy for MockRuntime {
